package ex2;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.microedition.media.MediaException;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.media.Manager;
import javax.microedition.media.Player;
import javax.microedition.media.PlayerListener;
import javax.microedition.media.control.RecordControl;

/**
 * This midlet demonstrates how a mobile phone's microphone can
 * be used to record audio. It also demonstrates how this data
 * can be played after it has been recorded.
 *
 * @author  Alexander De Luca, Gregor Broll
 */
public class AudioMidlet extends MIDlet implements CommandListener {
    private static final Command CLOSE_COMMAND = new Command("close", Command.EXIT, 1);
    private static final Command START_COMMAND = new Command("start",Command.SCREEN, 1);
    private static final Command PLAY_COMMAND = new Command("play", Command.SCREEN, 1);
    private static final Command STOP_COMMAND = new Command("stop",Command.SCREEN, 1);
    private static final Command BACK_COMMAND = new Command("back",Command.SCREEN, 1);
    private StringItem audioItem = new StringItem("recorded: ","0s");
    private boolean running = false;
    private Form audioForm;
    private Display display;
    private Player player;
    private RecordControl rc;
    private ByteArrayOutputStream output;
    private RecordThread recordThread;
    
    private void createForm() {
        display = Display.getDisplay(this);
        audioForm = new Form("Microphone");
        audioForm.append(audioItem);
        audioForm.addCommand(CLOSE_COMMAND);
        audioForm.addCommand(START_COMMAND);
        audioForm.setCommandListener(this);
        display.setCurrent(audioForm);
    }
    
    /**
     * Record audio.
     */
    private void recordAudio() {
        synchronized(this) {
            audioForm.removeCommand(START_COMMAND);
            audioForm.addCommand(STOP_COMMAND);
            if(!running) {
                this.running = true;
                recordThread = new RecordThread();
                recordThread.start();
            }
        }
    }
    
    /**
     * Stops the record and displays the result.
     */
    private void stopRecord() {
        if(!running) // ignore while not running
            return;
        
        recordThread.interrupt();
        this.running = false;
        audioItem.setText("0s");
        try {
            rc.commit();
            // save the recordedData in a byte array
            byte[] soundData = output.toByteArray();
            // close the player
            player.close();
            this.playSound(soundData);
            audioForm.removeCommand(STOP_COMMAND);
            audioForm.addCommand(START_COMMAND);
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    /**
     * Starts the audio playing.
     *
     * @param soundData The recorded byte array that includes the audio data.
     */
    private void playSound(byte[] soundData) {
        display.setCurrent(new AudioPlayer(soundData));
    }
    
    public void commandAction(Command comm, Displayable disp) {
        if(comm == CLOSE_COMMAND)
            this.exitMIDlet();
        else if(comm == START_COMMAND) {
            this.recordAudio();
        } else if(comm == STOP_COMMAND) {
            this.stopRecord();
        } else if(comm == BACK_COMMAND) {
            display.setCurrent(audioForm);
        }
    }
    
    /**
     * Free resources and close the midlet.
     */
    public void exitMIDlet() {
        player.close();
        display.setCurrent(null);
        destroyApp(true);
        notifyDestroyed();
    }
    
    public void startApp() {
        this.createForm();
    }
    
    public void pauseApp() {
    }
    
    public void destroyApp(boolean unconditional) {
    }
    
    /**
     * Inner class representing an audio player.
     */
    private class AudioPlayer extends Form implements CommandListener, PlayerListener {
        private byte[] audioData;
        private Player p2;
        private StringItem playItem = new StringItem("Press play to start", null);
		
        public AudioPlayer(byte[] audioData) {
            super("Play audio");
            this.audioData = audioData;
            this.append(playItem);
            this.addCommand(PLAY_COMMAND);
            this.addCommand(BACK_COMMAND);
            this.setCommandListener(this);
        }
		
        private void playSound() {
            new Thread() {
                public void run() {
                    ByteArrayInputStream audioInputStream = new ByteArrayInputStream(audioData);
                    try {
                        p2 = Manager.createPlayer(audioInputStream,"audio/x-wav");
                        p2.prefetch();
                        p2.addPlayerListener(AudioPlayer.this);
                        p2.start();
                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (MediaException e) {
                        e.printStackTrace();
                    }
                }
            }.start();
        }
		
        private void stopSound() {
            new Thread() {
                public void run() {
                    try {	
                        p2.stop();
                    } catch (MediaException e) {
                        e.printStackTrace();
                    }
                }
            }.start();
        }
        
        public void commandAction(Command comm, Displayable disp) {
            if(comm == BACK_COMMAND) {
                display.setCurrent(audioForm);
            } else if(comm == PLAY_COMMAND) {
                playSound();
            }else if(comm == STOP_COMMAND) {
                stopSound();
            }
        }

        public void playerUpdate(Player player, String event, Object eventData) {
            if(event.equals("started")) {
                this.removeCommand(PLAY_COMMAND);
                this.addCommand(STOP_COMMAND);
                playItem.setLabel("playing");
            } else if(event.equals("stopped")) {
                this.removeCommand(STOP_COMMAND);
                this.addCommand(PLAY_COMMAND);
                playItem.setLabel("stopped");
            } else if(event.equals("endOfMedia")) {
                this.removeCommand(STOP_COMMAND);
                this.addCommand(PLAY_COMMAND);
                playItem.setLabel("end");
            }
        }
   }

    private class RecordThread extends Thread {
        public RecordThread() {
            
        }
        
        public void run() {
            try {
                // create the player using standard encoding
                player = Manager.createPlayer("capture://audio");
                player.realize();
                // get the RecordControl over this Player
                rc = (RecordControl)player.getControl("RecordControl");
                // create an OutputStream which the RecordControl will use
                // to write write the recorded data.
                output = new ByteArrayOutputStream();
                rc.setRecordStream(output);
                // start the recording
                rc.startRecord();
                player.start();
                int seconds = 0;
                while(true) {
                    try {
                        sleep(1000);
                    } catch(InterruptedException e) {
                        System.out.println(e.toString());
                    }
                    seconds++;
                    audioItem.setText(Integer.toString(seconds)+"s");
                }
            } catch(MediaException e) {
                System.out.println(e.toString());
            } catch(IOException e) {
                System.out.println(e.toString());
            }
        }
    }

}
